/*
 * MINIGUI - Harbour Win32 GUI library Demo
 *
 * Copyright 2019 Verchenko Andrey <verchenkoag@gmail.com>
 * Copyright 2019 Sergej Kiselev <bilance@bilance.lv>
*/

#include "minigui.ch"
#include "TSBrowse.ch"

MEMVAR oBrw

STATIC cStaticMainFolder, aStatDirFrom, aStatDirTo, aStatDir3, aStatBtnClr 

Function Main
   Local cTitle := "The MAIN form of the program"

   SET OOP ON

   PRIVATE oBrw  // for Tsbrowse object

   //     / given as an example
   MyInitForm()

   DEFINE WINDOW Form_Main AT 0,0 WIDTH 640 HEIGHT 480 ;
       TITLE cTitle                                    ;
       ICON "1_MAIN"                                   ;
       MAIN                                            ;	
       ON INIT             {|| NIL }                   ; //    
       ON RELEASE          {|| NIL }                   ; //     
       ON INTERACTIVECLOSE {|| MyExit(.F.)  }            //    [x]

       //     / given as an example
       Addition_MainForms()

       oBrw:SetFocus()
                   
   END WINDOW

   CENTER   WINDOW Form_Main
   ACTIVATE WINDOW Form_Main

   Return Nil

/////////////////////////////////////////////////////////////////////
//      ()
// Initialize variables for my program (example)
INIT PROCEDURE MyInitWin()
   LOCAL o   

   SET CODEPAGE TO RUSSIAN
   SET LANGUAGE TO RUSSIAN

   //RDDSETDEFAULT('DBFCDX')

   SET CENTURY      ON
   SET DATE         GERMAN
   SET DELETED      ON
   SET EXCLUSIVE    ON
   SET EPOCH TO    ( Year(Date()) - 50 )
   SET AUTOPEN      ON
   SET EXACT        ON
   SET SOFTSEEK     ON

   SET NAVIGATION   EXTENDED
   SET FONT         TO "Arial", 14
   SET DEFAULT ICON TO "hmg_ico"

   SET BROWSESYNC ON                 //  browse   recno
   SET CENTERWINDOW RELATIVE PARENT  //       

   // --------------------------------
   SET OOP ON
   // --------------------------------

   aStatBtnClr := WHITE
   cStaticMainFolder := GetStartUpFolder()+"\"

   //      , .. SET OOP ON !!!
   App.Cargo := oKeyData()  //    App.Cargo:< >            

   o := App.Cargo           //     

   //    App.Cargo.
   //   ,     NIL
   o:BColorGet      := {{255,255,255},{255,255,200},{200,255,255}}
   o:FColorGet      := {{0,0,0},{255,255,200},{0,0,255}}          
   o:BColor0        := {  0,155,173}  // color of the form window
   o:BColor1        := {  0,176,240}
   o:BColorGetFocus := o:BColorGet[3]
   o:FColorGetFocus := o:FColorGet[3]

   o:aBackColorOffice_2003 := {159,191,236}   // Office_2003 Blue   

   o:aGradOverBX           := { { 0.5, CLR_YELLOW, CLR_WHITE } , ;
                                { 0.5, CLR_WHITE , CLR_YELLOW } }
   o:aGradFillB1           := { { 0.5, CLR_OK    , CLR_WHITE } , ;
                                { 0.5, CLR_WHITE , CLR_OK } } 
   O:aGradFillBE           := { { 0.5, CLR_RED   , CLR_WHITE }, ;
                                { 0.5, CLR_WHITE , CLR_RED } }
   
   SET GETBOX FOCUS BACKCOLOR TO o:BColorGetFocus    //   getbox
   SET GETBOX FOCUS FONTCOLOR TO o:FColorGetFocus    // .  

RETURN

////////////////////////////////////////////////////////////
//     - Exit programm
STATIC FUNCTION MyExit( lClose )          
   LOCAL cMsg, lExit, aDim

   aDim := FormDarkenSave( ThisWindow.Name , , ) //  /    
                                                 // remember the colors/background of the objects on the form

   cMsg := ';'+SPACE(10)+'Do you really want to exit? '+SPACE(10)+' ; ;'
   cMsg := AtRepl( ";", cMsg, CRLF )
   lExit := MsgYesNo( cMsg, "Exit", .F.,, .F. )
   IF lExit .AND. lClose
      Form_Main.Release
   ENDIF

   FormDarkenRestore( aDim ) //  /           
                             // restore colors / background of objects on the form
RETURN lExit

///////////////////////////////////////////////////////////
//     
// This function initializes the data on the form
STATIC FUNCTION MyInitForm()
   LOCAL aArray, aHead, aWSize, aFoot, aPict, aAlign, aName, aFontHF
   LOCAL aDim, nI, cStr, cPath := cStaticMainFolder
 
   //     / given as an example
   cStr := hb_compiler() + CRLF + Version() + CRLF + MiniGuiVersion() + CRLF
   HB_MemoWrit( cPath + "FormDarken(0.7).log", cStr )

   aStatDirFrom := {}
   aDim := Directory( cPath + "*.prg" )
   IF Len( aDim ) > 0
      FOR nI := 1 TO Len( aDim )
         AAdd( aStatDirFrom, aDim[ nI, 1 ] )
      NEXT
   ENDIF

   aStatDirTo := {}
   aDim := Directory( cPath + "*.exe" )
   IF Len( aDim ) > 0
      FOR nI := 1 TO Len( aDim )
         AAdd( aStatDirTo, aDim[ nI, 1 ] )
      NEXT
   ENDIF

   //      
   aWSize  := NIL ; aFoot := NIL ; aPict := NIL ; aAlign := NIL ; aFontHF := NIL
   aHead   := {""     , "File"  , "Size"  , "Date"  , "Time"   }
   aName   := {"Name_1" , "Name_2", "Name_3", "Name_4", "Name_5" }

   aDim    := Directory( cPath + "*.*" )
   aArray  := {}
   IF Len( aDim ) > 0
      FOR nI := 1 TO Len( aDim )
         AADD( aArray, { nI, aDim[nI,1], aDim[nI,2], aDim[nI,3], aDim[nI,4], aDim[nI,5] } ) 
      NEXT
   ELSE
      //      / always create one array line
      AADD( aArray, { 0, "", 0, CTOD(""), "" } ) 
   ENDIF

   aStatDir3 := { aArray, aHead, aWSize, aFoot, aPict, aAlign, aName, aFontHF }

   RETURN Nil

/////////////////////////////////////////////////////////////////////////
//      
// Function additional drawings main task form
STATIC FUNCTION Addition_MainForms()    
   LOCAL cTitle := "An example of shading / restoring shape color after selecting"
   LOCAL aGradFillB1, aGradFillBE, aGradOverBX, aGroup := {}
   LOCAL nFrmWidth, nFrmHeight, aDatos, aDim, nRow, nWTwo
   LOCAL nRBrw, nCBrw, nWBrw, nHBrw, nBrwFSize, aBrwBackColor
   LOCAL nHBut, nRBut, nWBut, nCBut, nHList, nRList, nWList, nRCbox

   This.Width  := 900   //     
   This.Height := 640   // set the new size of the form window

   This.Sizable   := .F.  // NOSIZE
   This.MaxButton := .F.  // NOMAXIMIZE

   This.BackColor := App.Cargo:BColor0  //{ 0,155,173}  // color of the form window

   nFrmWidth  := This.ClientWidth 
   nFrmHeight := This.ClientHeight

   //     / add an object to a form
   @ 10,0 LABEL Label_1 VALUE cTitle ; // PARENT Form_Main ;
     WIDTH nFrmWidth HEIGHT 48 SIZE 22 BOLD ;
     FONTCOLOR WHITE TRANSPARENT CENTERALIGN

   /////////////////////// Building ListBox  ////////////////////////////
   nHBut   := 125   //  
   nRList  := 60
   nWList  := 155   //   
   nHList  := nFrmHeight - nRList - nHBut - 20*2

   DEFINE LISTBOX List_1
      ROW    nRList
      COL    20
      WIDTH  nWList
      HEIGHT nHList
      ITEMS  AMERGE(aStatDirFrom, aStatDirTo) //   
      FONTSIZE 12
      FONTBOLD .T.
      FONTCOLOR BLUE
      BACKCOLOR App.Cargo:BColor1  //{0,176,240}
      ONDBLCLICK {|| aDim := FormDarkenSave(,,,2) ,;
                     SayValueObj(0)               ,;    
                     FormDarkenRestore(aDim,2)    ,;
                     oBrw:SetFocus()               }
   END LISTBOX
   /////////////////////// Building ListBox  ///////////////////////////
   
   @ nRList, 200 IMAGE Image_1 PICTURE 'DEMO' ;
     WIDTH 155 HEIGHT 84 STRETCH TRANSPARENT  ;
     ACTION MsgInfo(MiniGuiVersion()) OnMouseHover RC_CURSOR( "MINIGUI_FINGER" )

   /////////////////////// Building CheckBox  ///////////////////////////
   nWTwo  := This.Image_1.Width
   nRCbox := nRList + This.Image_1.Height + 20
   @ nRCbox, 200 CHECKBOX  CheckBox_1                  ; 
     CAPTION "CheckBox 1" VALUE .T.                    ;
     WIDTH nWTwo HEIGHT 24 FONTCOLOR BLACK TRANSPARENT          

   nRCbox := This.CheckBox_1.Row + This.CheckBox_1.Height + 10
   @ nRCbox, 200 CHECKBOX  CheckBox_2                  ; 
     CAPTION "CheckBox 2" VALUE .F.                    ;
     WIDTH nWTwo HEIGHT 24 FONTCOLOR BLACK TRANSPARENT          
   /////////////////////// Building CheckBox  ///////////////////////////

   nRow := This.CheckBox_2.Row + This.CheckBox_2.Height + 10
   @ nRow, 200 COMBOBOXEX ComboEx_1            ; 
     WIDTH nWTwo HEIGHT 390                    ;
     ITEMS AMERGE(aStatDirFrom, aStatDirTo)    ; 
     FONT 'Tahoma' SIZE 14  VALUE 1            ;
     BACKCOLOR App.Cargo:BColor1               ; 
     TOOLTIP "Sample help ComboEx_1"  

   nRow := This.ComboEx_1.Row + 20*2
   @ nRow, 200 TEXTBOX TextBox_1               ;
     WIDTH nWTwo HEIGHT 30                     ;
     VALUE "TextBox 1" FONT 'Tahoma' SIZE 14   ;
     FONTCOLOR RED BACKCOLOR App.Cargo:BColor1 ;  
     TOOLTIP "Sample help TextBox_1"  

   nRow := This.TextBox_1.Row + This.TextBox_1.Height + 10
   @ nRow, 200 GETBOX GetBox_1                 ;
     HEIGHT 30 WIDTH nWTwo                     ;
     VALUE DATE() PICTURE '@D'                 ;
     BACKCOLOR App.Cargo:BColorGet             ;
     FONTCOLOR App.Cargo:FColorGet             ;
     TOOLTIP "Sample help GetBox_1"  

   nRow := This.GetBox_1.Row + This.GetBox_1.Height + 10
   @ nRow, 200 FRAME Frame_1                   ; 
     CAPTION " Frame " WIDTH nWTwo HEIGHT 90 TRANSPARENT

   AADD( aGroup , "RadioGroup-1" )
   AADD( aGroup , "RadioGroup-2" )

   nRow := This.Frame_1.Row + 25
   @ nRow, 200 + 15 RadioGroup RG_1  ; 
     Options aGroup VALUE 1          ;
     Width nWTwo - 20*2 Spacing 30   ;
     FONT 'Tahoma' SIZE 12 BOLD      ;
     FONTCOLOR BLUE TRANSPARENT       

   /////////////////////// Building Browse  ////////////////////////////
   nRBrw := This.List_1.Row 
   nCBrw := This.Image_1.Col + This.Image_1.Width + 20
   nWBrw := nFrmWidth - nCBrw - 20
   nHBrw := This.List_1.Height
   nBrwFSize := 12
   aDatos := aStatDir3
   aBrwBackColor := App.Cargo:aBackColorOffice_2003  //   

   DEFINE TBROWSE oBrw              ;
      AT nRBrw, nCBrw               ; 
      WIDTH  nWBrw  HEIGHT nHBrw    ;
      BACKCOLOR aBrwBackColor       ;
      FONT 'Tahoma' SIZE nBrwFSize  ;
      GRID
      //   
      TsbCreate(oBrw, aDatos, nBrwFSize, aBrwBackColor ) 
   END TBROWSE 
   /////////////////////// Building Browse  ////////////////////////////


   /////////////////////// Building Button  ////////////////////////////
   nHBut := 125   //  
   nRBut := nFrmHeight - nHBut - 20
   nWBut := ( nFrmWidth - 20*5 ) / 4    // 4   
   nCBut := 20

   aGradOverBX := App.Cargo:aGradOverBX
   aGradFillB1 := App.Cargo:aGradFillB1

   //     / add an object to a form
   @ nRBut, nCBut BUTTONEX BUTTON_1                              ;
      WIDTH nWBut HEIGHT nHBut CAPTION "Sample 1" ICON "iPiople" ;
      NOXPSTYLE VERTICAL HANDCURSOR TOOLTIP "Button_1"           ;
      FONTCOLOR aStatBtnClr SIZE 16 BOLD                         ;
      BACKCOLOR aGradOverBX  GRADIENTFILL aGradFillB1            ;
      ON MOUSEHOVER ( This.Fontcolor := GRAY     , This.Icon := "iPiople2", This.GradientFill := aGradFillB1 ) ; 
      ON MOUSELEAVE ( This.Fontcolor := aStatBtnClr, This.Icon := "iPiople" , This.GradientOver := aGradOverBX ) ; 
      ACTION {|| aDim := FormDarkenSave( ThisWindow.Name ) ,;
               SayValueObj(0) ,; // MsgDebug( ThisWindow.Name , This.Name )     ,;
               FormDarkenRestore( aDim )                   ,;
               oBrw:SetFocus() }

   nCBut := 20*2 + nWBut
   @ nRBut, nCBut BUTTONEX BUTTON_2                              ;
      WIDTH nWBut HEIGHT nHBut CAPTION "Sample 2" ICON "iMusic"  ;
      NOXPSTYLE VERTICAL HANDCURSOR TOOLTIP "Button_2"           ;
      FONTCOLOR aStatBtnClr BACKCOLOR LGREEN SIZE 16 BOLD        ;
      ON MOUSEHOVER ( This.Fontcolor := GRAY       , This.Icon := "iMusic2", This.Backcolor := YELLOW ) ; 
      ON MOUSELEAVE ( This.Fontcolor := aStatBtnClr, This.Icon := "iMusic" , This.Backcolor := LGREEN ) ; 
      ACTION {|| aDim := FormDarkenSave( ThisWindow.Name , GRAY , GRAY ) ,;
               SayValueObj(1), ; // MsgDebug( ThisWindow.Name , This.Name )   ,;
               FormDarkenRestore( aDim )                 ,;
               oBrw:SetFocus() }

   nCBut := 20*3 + nWBut*2
   @ nRBut, nCBut BUTTONEX BUTTON_3                             ;
      WIDTH nWBut HEIGHT nHBut CAPTION "Sample 3" ICON "iAbout" ;
      NOXPSTYLE VERTICAL HANDCURSOR TOOLTIP "Button_3"          ;
      FONTCOLOR aStatBtnClr BACKCOLOR PURPLE SIZE 16 BOLD       ;
      ON MOUSEHOVER ( This.Fontcolor := GRAY     , This.Icon := "iAbout2", This.Backcolor := YELLOW ) ; 
      ON MOUSELEAVE ( This.Fontcolor := aStatBtnClr, This.Icon := "iAbout" , This.Backcolor := PURPLE ) ; 
      ACTION {|| aDim := FormDarkenSave( ThisWindow.Name , BLACK , BLACK ) ,;
               SayValueObj(2), ; // MsgDebug( ThisWindow.Name , This.Name )    ,;
               FormDarkenRestore( aDim )                  ,;
               oBrw:SetFocus() }

   nCBut := 20*4 + nWBut*3
   aGradFillBE := App.Cargo:aGradFillBE
   @ nRBut, nCBut BUTTONEX BUTTON_Exit                       ; 
      WIDTH nWBut HEIGHT nHBut CAPTION "Exit" ICON "iExit"   ;
      NOXPSTYLE VERTICAL HANDCURSOR TOOLTIP "Button_Exit"    ;
      FONTCOLOR aStatBtnClr SIZE 16 BOLD                     ;
      BACKCOLOR aGradOverBX  GRADIENTFILL aGradFillBE        ;
      ON MOUSEHOVER ( This.Fontcolor := GRAY       , This.Icon := "iExit2", This.GradientFill := aGradFillBE ) ; 
      ON MOUSELEAVE ( This.Fontcolor := aStatBtnClr, This.Icon := "iExit" , This.GradientOver := aGradOverBX ) ; 
      ACTION {|| MyExit(.T.) , oBrw:SetFocus() }

   /////////////////////// Building Button  ////////////////////////////

   RETURN NIL

/////////////////////////////////////////////////////////////////////////
STATIC FUNCTION SayValueObj( nMode, cName, lEque )
   LOCAL oWnd := ThisWindow.Object  //  
   LOCAL oCtl := This.Object        //   
   LOCAL aRet := {}, aObj
   LOCAL cTxt := '', t, o
   DEFAULT nMode := 0
? "-------------" + PROCNAME(0) + ":" + HB_NtoS(PROCLINE(0)), "nMode=", nMode
? 'Form   :', oWnd:Type, oWnd:Name, oWnd:Handle, oWnd:Index, ;
              oWnd:IsWindow, oWnd:IsControl, ;
              oWnd:Row , oWnd:Col , oWnd:Width , oWnd:Height, ;
              oWnd:ClientWidth , oWnd:ClientHeight
//   
// oWnd:Hide(), oWnd:Show(), oWnd:Restore(), oWnd:SetFocus(ControlName)
// oWnd:SetSize(y,x,w,h), oWnd:GetObj(ControlName), oWnd:Release()

? 'Control:', oCtl:Type, oCtl:Name, oCtl:Handle, oCtl:Index, ;
              oCtl:IsWindow, oCtl:IsControl, ;
              oCtl:Row , oCtl:Col , oCtl:Width , oCtl:Height, ;
              oCtl:ClientWidth , oCtl:ClientHeight
//  
// xVal := oCtl:Value
// oCtl:Value := xVal
// oCtl:Hide(), oCtl:Show(), oCtl:SetSize(y,x,w,h), oCtl:SetFocus()
// oCtl:Refresh(), oCtl:Disable(), oCtl:Enable()
// oGet := oCtl:Get -  GetBox
// oBrw := oCtl:Tsb -  TsBrowse
// oBrw:Refresh()
// oBrw:...  ..

//   H_OBJECTS.PRG

   If     nMode == 1
       DEFAULT cName := 'BUTT,RG_'
       aRet := oWnd:GetObj4Name( cName )
       cTxt += 'Control [' + cName + '] property ( by name ) for window :'
// aRet -   
       FOR EACH o IN aRet        
           oWnd := o:Window
           oCtl := o 
           If ! _IsControlDefined(oCtl:Name, oWnd:Name)
              LOOP
           EndIf
? 'Form   :', oWnd:Type, oWnd:Name, oWnd:Handle, oWnd:Index, oWnd:IsWindow, ;
              oWnd:Row , oWnd:Col , oWnd:Width , oWnd:Height, ;
              oWnd:ClientWidth , oWnd:ClientHeight
? 'Control:', oCtl:Type, oCtl:Name, oCtl:Handle, oCtl:Index, oCtl:IsWindow, ;
              oCtl:Row , oCtl:Col , oCtl:Width , oCtl:Height, ;
              oCtl:ClientWidth , oCtl:ClientHeight

           cTxt += CRLF + o:Type + '  ' + o:Name + '  ' + o:Window:Name + ;
                                                   '  ' + hb_ntos(o:Index)
        // ..    / those. can do
        // SetProperty(o:Window:Name, o:Name, 'Enabled', .F. )
        // SetProperty(o:Window:Name, o:Name, 'Enabled', .T. )
        // DoMethod   (o:Window:Name, o:Name, 'Hide'         )
        // DoMethod   (o:Window:Name, o:Name, 'Show'         )
        // ...
       NEXT
   ElseIf nMode == 2
       DEFAULT cName := 'LAB,GET,BUTT'
       aRet := oWnd:GetObj4Type( cName, lEque )
// aRet -   
       cTxt += 'Control [' + cName + '] property ( by type ) for window :'
       FOR EACH o IN aRet
           oWnd := o:Window
           oCtl := o 
           If ! _IsControlDefined(oCtl:Name, oWnd:Name)
              LOOP
           EndIf
? 'Form   :', oWnd:Type, oWnd:Name, oWnd:Handle, oWnd:Index, oWnd:IsWindow, ;
              oWnd:Row , oWnd:Col , oWnd:Width , oWnd:Height, ;
              oWnd:ClientWidth , oWnd:ClientHeight
? 'Control:', oCtl:Type, oCtl:Name, oCtl:Handle, oCtl:Index, oCtl:IsWindow, ;
              oCtl:Row , oCtl:Col , oCtl:Width , oCtl:Height, ;
              oCtl:ClientWidth , oCtl:ClientHeight

           cTxt += CRLF + o:Type + '  ' + o:Name + '  ' + o:Window:Name + ;
                                                   '  ' + hb_ntos(o:Index)
        // ..    / those. can do
        // SetProperty(o:Window:Name, o:Name, 'Enabled', .F. )
        // SetProperty(o:Window:Name, o:Name, 'Enabled', .T. )
        // DoMethod   (o:Window:Name, o:Name, 'Hide'         )
        // DoMethod   (o:Window:Name, o:Name, 'Show'         )
        // ...
       NEXT
   Else
       aRet := oWnd:GetListType()
// aRet -    , .. 
       cTxt += 'All Control type for window :'
       FOR EACH t IN aRet     // t -  
           cTxt += CRLF + t
           // ..    / those. can do
           aObj := oWnd:GetObj4Type( t )
// aObj -     t
           FOR EACH o IN aObj // o -  
               oWnd := o:Window
               oCtl := o 
               If ! _IsControlDefined(oCtl:Name, oWnd:Name)
                  LOOP
               EndIf
? oWnd, oCtl, _IsControlDefined(oCtl:Name, oWnd:Name)
? 'Form   :', oWnd:Type, oWnd:Name, oWnd:Handle, oWnd:Index, oWnd:IsWindow, ;
              oWnd:Row , oWnd:Col , oWnd:Width , oWnd:Height, ;
              oWnd:ClientWidth , oWnd:ClientHeight
? 'Control:', oCtl:Type, oCtl:Name, oCtl:Handle, oCtl:Index, oCtl:IsWindow, ;
              oCtl:Row , oCtl:Col , oCtl:Width , oCtl:Height, ;
              oCtl:ClientWidth , oCtl:ClientHeight

        // SetProperty(o:Window:Name, o:Name, 'Enabled', .F. )
        // SetProperty(o:Window:Name, o:Name, 'Enabled', .T. )
        // DoMethod   (o:Window:Name, o:Name, 'Hide'         )
        // DoMethod   (o:Window:Name, o:Name, 'Show'         )
        // ...
           NEXT
       NEXT
   EndIf

   If ! empty(cTxt)
      cTxt += CRLF + CRLF + ThisWindow.Name + CRLF + This.Name  
      cTxt += CRLF + CRLF + (ThisWindow.Object):Name + CRLF + (This.Object):Name  
//
// ? (This.Object):Row, (This.Object):Col, (This.Object):Width, (This.Object):Height
//
      oWnd := ThisWindow.Object  
      oCtl := This.Object        
      cTxt += CRLF + CRLF + oWnd:Type + '  ' + oWnd:Name + CRLF + ;
                            oCtl:Type + '  ' + oCtl:Name  
      HMG_Alert( cTxt, , ProcName() )
   EndIf
   
RETURN NIL
